/************************************************************
* $Id: main.c,v 1.2 2014/11/17 11:16:55 mechanoid Exp mechanoid $
*
*       OS : GNU/Linux 3.14.19 x86_64
* COMPILER : GNU project C and C++ Compiler  v. 4.9.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://www.mechanoid.kiev.ua
*  e-mail : cv@mechanoid.kiev.ua
* 
************************************************************/
#include <stdio.h>
#include <cv.h>
#include <highgui.h>

#define WIN_SRC   "ocv-example original"
#define WIN_CTL   "ocv-example control"
#define WIN_RES   "ocv-example result"

#define T 128

IplImage *frame_c1[2];

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]);
int run(char* argv[]);
void done();

void update(int pos, void* userdata);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main(int argc, char* argv[]) {
   if(!init(argc,argv)) { run(argv); }
   done();
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]) {
   int frame_w;
   int frame_h;

   if(argc<3) { return 1; }

   // получаем картинку
   frame_c1[0] = cvLoadImage(argv[1],CV_LOAD_IMAGE_GRAYSCALE);
   if( !frame_c1[0]) { fprintf(stderr,"[E] cvLoadImage\n"); return 1; }

   frame_w=frame_c1[0]->width;
   frame_h=frame_c1[0]->height;
   fprintf(stdout,"[i] %ix%i \n",frame_w,frame_h);

   frame_c1[1] = cvCloneImage(frame_c1[0]);
   if( !frame_c1[1]) { fprintf(stderr,"[E] cvCloneImage\n"); return 1; }

   // открываем и размещаем окна
   cvNamedWindow(WIN_SRC, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow(WIN_SRC, 10,20);
   
   cvNamedWindow(WIN_RES, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow(WIN_RES, 10+(10+frame_w),20);

   cvNamedWindow(WIN_CTL, CV_WINDOW_NORMAL); 
   cvResizeWindow(WIN_CTL, 150, 20);
   cvMoveWindow(WIN_CTL, 10,20+(20+frame_h));

   cvCreateTrackbar("T: ", WIN_CTL, NULL, 253, (CvTrackbarCallback)update);

   fprintf(stdout,"[i] press any key to exit...\n");

   return 0;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void done(){
   cvDestroyAllWindows();
   cvReleaseImage(&frame_c1[0]);
   cvReleaseImage(&frame_c1[1]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int run(char* argv[]) { 
   cvShowImage( WIN_SRC, frame_c1[0]); 
   update(T,NULL);
   cvWaitKey(0);
   cvSaveImage(argv[2],frame_c1[1],0);
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void update(int pos, void* userdata){
   cvThreshold( frame_c1[0], frame_c1[1], pos+1, 255.0, CV_THRESH_BINARY ); 
   fprintf(stdout,"[i] T: %i\n",pos);
   cvShowImage( WIN_RES, frame_c1[1]); 
}


