/************************************************************
* $Id: main.c,v 1.5 2014/11/17 13:50:40 mechanoid Exp $
*
*       OS : GNU/Linux 3.14.19 x86_64
* COMPILER : GNU project C and C++ Compiler  v. 4.9.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://www.mechanoid.kiev.ua
*  e-mail : cv@mechanoid.kiev.ua
* 
************************************************************/

#include <cv.h>
#include <highgui.h>
#include <stdio.h>

#define WIN_SRC   "ocv-example original"
#define WIN_ALV   "ocv-example autolevel"
#define WIN_GRW   "ocv-example grayworld"

IplImage* frame_c3[2]; 
IplImage* frame_c1[3]; 

IplImage* frame_c3f[2] ;
IplImage* frame_c1f[3] ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char *argv[]);
int run(char *argv[]);
void done();

int grayworld(IplImage* src,IplImage* dst); 
int autolevels(IplImage* src,IplImage* dst); 


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main(int argc, char *argv[]) {
   if(!init(argc,argv)){ run(argv);}
   done();
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char *argv[]) { 
   int frame_w;
   int frame_h;
   int i;

   if(argc<3) { fprintf( stderr, "[E] bad parameters\n\n\t%s <in> <out>\n",argv[0]); return 1; }

   frame_c3[0] = cvLoadImage( argv[1], CV_LOAD_IMAGE_COLOR);
   if(!frame_c3[0]){ fprintf( stderr, "[E] cvLoadImage %s \n",argv[1]); return 3; }
  
   frame_w=frame_c3[0]->width;
   frame_h=frame_c3[0]->height;
   fprintf(stdout,"[i] %ix%i \n", frame_w, frame_h);

   frame_c3[1] = cvCloneImage(frame_c3[0]);
   if(!frame_c3[1]){ fprintf( stderr, "[E] cvCloneImage\n"); return 3; }

   for(i=0;i<3;i++){
	  frame_c1[i]= cvCreateImage(cvGetSize(frame_c3[0]),frame_c3[0]->depth,1);
	  if(!frame_c1[i]){ fprintf( stderr, "[E] cvCreateImage\n"); return 4; }		
   }

   for(i=0;i<2;i++) {
	  frame_c3f[i] = cvCreateImage(cvGetSize(frame_c3[0]),IPL_DEPTH_32F,3);
	  if(!frame_c3f[i]){ fprintf( stderr, "[E] cvCreateImage\n"); return 5; }
	  cvZero(frame_c3f[i]);
   }

   for(i=0;i<3;i++) {
	  frame_c1f[i] = cvCreateImage(cvGetSize(frame_c3[0]),IPL_DEPTH_32F,1);
	  if(!frame_c1f[i]){ fprintf( stderr, "[E] cvCreateImage\n"); return 5; }
	  cvZero(frame_c1f[i]);
   }

   
   cvNamedWindow( WIN_SRC, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_SRC, 10,30 );

   cvNamedWindow( WIN_ALV, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_ALV, 10+(10+frame_w)*1,30);

   cvNamedWindow( WIN_GRW, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_GRW, 10,30+(30+frame_h));


   fprintf(stdout,"[i] press any key to exit...\n");


   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int run(char *argv[]) { 
   
   cvShowImage(WIN_SRC, frame_c3[0]); // отображаем оригинал

   autolevels(frame_c3[0],frame_c3[1]);
   cvShowImage(WIN_ALV, frame_c3[1]);
  
   grayworld(frame_c3[0],frame_c3[1]);
   cvShowImage(WIN_GRW, frame_c3[1]);

   //cvSaveImage(argv[2], frame_c3[1], 0 );
   
   cvWaitKey(0);

   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void done() { 
   int i;

   cvDestroyAllWindows();
   
   for(i=0;i<2;i++){ cvReleaseImage( &frame_c3[i] ); }
   for(i=0;i<3;i++){ cvReleaseImage( &frame_c1[i] ); }

   for(i=0;i<2;i++) { cvReleaseImage( &frame_c3f[i] ); }
   for(i=0;i<3;i++) { cvReleaseImage( &frame_c1f[i] ); }
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// grayworld -- выравниваем среднее значение по каналам
int grayworld(IplImage* src,IplImage* dst) {
   double f=0.0;
   CvScalar av;

   cvConvert( src, frame_c3f[0] );

   av=cvAvg(frame_c3f[0],NULL); // вычисляем средние яркости по каналам
   f=(av.val[0]+av.val[1]+av.val[2])/3.0; // вычисляем среднюю яркость
   // коэффициенты для коррекции яркостей по каналам
   av.val[0]=f/av.val[0];
   av.val[1]=f/av.val[1];
   av.val[2]=f/av.val[2];

   // корректируем яркость по каналам
   cvSplit( frame_c3f[0], frame_c1f[0], frame_c1f[1], frame_c1f[2], NULL);
   cvConvertScale(frame_c1f[0],frame_c1f[0],av.val[0],0.0);		
   cvConvertScale(frame_c1f[1],frame_c1f[1],av.val[1],0.0);		
   cvConvertScale(frame_c1f[2],frame_c1f[2],av.val[2],0.0);		
   cvMerge( frame_c1f[0], frame_c1f[1], frame_c1f[2],  NULL, frame_c3f[0]);

   cvConvert( frame_c3f[0], dst );


   return 0; 
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// autolevels -- Растяжение контрастности
int autolevels(IplImage* src,IplImage* dst) {
   cvSplit( src, frame_c1[0],frame_c1[1],frame_c1[2],NULL);
   cvNormalize(frame_c1[0], frame_c1[0], 0.0, 255.0, CV_MINMAX,NULL); 
   cvNormalize(frame_c1[1], frame_c1[1], 0.0, 255.0, CV_MINMAX,NULL); 
   cvNormalize(frame_c1[2], frame_c1[2], 0.0, 255.0, CV_MINMAX,NULL); 
   cvMerge( frame_c1[0],frame_c1[1],frame_c1[2], NULL,dst);
   return 0; 
}

