/************************************************************
* $Id: main.c,v 1.2 2014/11/18 06:20:43 mechanoid Exp $
*
*       OS : GNU/Linux 3.14.19 x86_64
* COMPILER : GNU project C and C++ Compiler  v. 4.9.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://www.mechanoid.kiev.ua
*  e-mail : cv@mechanoid.kiev.ua
* 
************************************************************/
#include <stdio.h>
#include <cv.h>
#include <highgui.h>

#define WIN_SRC  "ocv-example original"
#define WIN_RES  "ocv-example result"
#define WIN_CTL  "ocv-example control"

#define T 100

IplImage *frame_c3;
IplImage *frame_c3f[2] ;

int c=(int)(T*0.5-1.0);
int g=(int)(T*0.5-1.0);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]);
int run(char* argv[]);
void done();
void update(int pos, void* userdata);


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main(int argc, char* argv[]) {
   if(!init(argc,argv)) { run(argv); }
   done();
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void done(){
   int i;
   cvDestroyAllWindows();
   cvReleaseImage(&frame_c3);
   for( i=0 ; i<2 ; i++ ) cvReleaseImage(&frame_c3f[i]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]){
   int w=0;
   int h=0;

   if(argc<3) return 1;

    // получаем картинку
   frame_c3 = cvLoadImage(argv[1],CV_LOAD_IMAGE_COLOR);
   if( !frame_c3) { fprintf(stderr,"[E] cvLoadImage\n"); return 1; }

   // проверяем параметры картинки
   w=frame_c3->width;
   h=frame_c3->height;
   fprintf(stdout,"[i] %ix%i \n",w,h);

   frame_c3f[0] = cvCreateImage(cvSize(w,h),IPL_DEPTH_32F,frame_c3->nChannels);
   if(!frame_c3f[0]){ fprintf(stderr,"[E] cvCloneImage\n"); return 2; }

   frame_c3f[1] = cvCreateImage(cvSize(w,h),IPL_DEPTH_32F,frame_c3->nChannels);
   if(!frame_c3f[1]){ fprintf(stderr,"[E] cvCloneImage\n"); return 3; }

   // открываем и размещаем окна
   cvNamedWindow(WIN_SRC, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow(WIN_SRC, 10,20);
   
   cvNamedWindow(WIN_RES, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow(WIN_RES, 10+(10+w)*1,20);

   cvNamedWindow(WIN_CTL, CV_WINDOW_NORMAL ); 
   cvCreateTrackbar("g: ", WIN_CTL, &g, T, (CvTrackbarCallback)update);
   cvCreateTrackbar("c: ", WIN_CTL, &c, T, (CvTrackbarCallback)update);
   cvResizeWindow(WIN_CTL,250,20);
   cvMoveWindow(WIN_CTL, 10,(20+h)+20);

   fprintf(stdout,"[i] press any key to exit...\n");


   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int run(char* argv[]) { 
   cvShowImage( WIN_SRC, frame_c3); 
   cvConvert(frame_c3,frame_c3f[0] );
   update(0,NULL);
   cvWaitKey(0);
   
   cvSaveImage(argv[2], frame_c3, 0 );

   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void update(int pos, void* userdata){
   double gg=(g+1)*2.0*1.0/T;
   double cc=(c+1)*2.0*1.0/T;

   fprintf(stdout,"[i] g:%f\tc:%f\n",gg,cc);

   cvPow(frame_c3f[0], frame_c3f[1], gg);
   cvConvertScale(frame_c3f[1], frame_c3f[1],cc,0.0);
   cvConvert(frame_c3f[1], frame_c3);
   cvShowImage( WIN_RES, frame_c3); // отображаем результат
}



