/************************************************************
* $Id: main.c,v 1.5 2014/11/17 13:50:40 mechanoid Exp $
*
*       OS : GNU/Linux 3.14.19 x86_64
* COMPILER : GNU project C and C++ Compiler  v. 4.9.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://www.mechanoid.kiev.ua
*  e-mail : cv@mechanoid.kiev.ua
* 
************************************************************/

#include <cv.h>
#include <highgui.h>
#include <stdio.h>

#define WIN_SRC   "ocv-example original"
#define WIN_HEQ   "ocv-example histogram eq"

IplImage* frame_c3[2]; 
IplImage* frame_c1[3]; 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char *argv[]);
int run(char *argv[]);
void done();
int histogr_eq(IplImage* src,IplImage* dst); 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main(int argc, char *argv[]) {
   if(!init(argc,argv)){ run(argv);}
   done();
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char *argv[]) { 
   int frame_w;
   int frame_h;
   int i;

   if(argc<3) { fprintf( stderr, "[E] bad parameters\n\n\t%s <in> <out>\n",argv[0]); return 1; }

   frame_c3[0] = cvLoadImage( argv[1], CV_LOAD_IMAGE_COLOR);
   if(!frame_c3[0]){ fprintf( stderr, "[E] cvLoadImage %s \n",argv[1]); return 3; }
  
   frame_w=frame_c3[0]->width;
   frame_h=frame_c3[0]->height;
   fprintf(stdout,"[i] %ix%i \n", frame_w, frame_h);

   frame_c3[1] = cvCloneImage(frame_c3[0]);
   if(!frame_c3[1]){ fprintf( stderr, "[E] cvCloneImage\n"); return 3; }

   for(i=0;i<3;i++){
	  frame_c1[i]= cvCreateImage(cvGetSize(frame_c3[0]),frame_c3[0]->depth,1);
	  if(!frame_c1[i]){ fprintf( stderr, "[E] cvCreateImage\n"); return 4; }		
   }

     
   cvNamedWindow( WIN_SRC, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_SRC, 10,30 );

   cvNamedWindow( WIN_HEQ, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_HEQ, 10+(10+frame_w),30);

   fprintf(stdout,"[i] press any key to exit...\n");


   return 0;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void done() { 
   int i;

   cvDestroyAllWindows();
   for(i=0;i<2;i++){ cvReleaseImage( &frame_c3[i] ); }
   for(i=0;i<3;i++){ cvReleaseImage( &frame_c1[i] ); }
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int run(char *argv[]) { 
   
   cvShowImage(WIN_SRC, frame_c3[0]); // отображаем оригинал

   histogr_eq(frame_c3[0],frame_c3[1]);
   cvShowImage(WIN_HEQ, frame_c3[1]);

   cvWaitKey(0);

//   cvSaveImage(argv[2], frame_c3[1], 0 );

   return 0;
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int histogr_eq(IplImage* src,IplImage* dst) {
   cvSplit( src, frame_c1[0],frame_c1[1],frame_c1[2],NULL);
   cvEqualizeHist(frame_c1[0],frame_c1[0]);
   cvEqualizeHist(frame_c1[1],frame_c1[1]);
   cvEqualizeHist(frame_c1[2],frame_c1[2]);
   cvMerge( frame_c1[0],frame_c1[1],frame_c1[2], NULL, dst );
 
   return 0;
}
