/************************************************************
* $Id: main.c,v 1.3 2014/11/16 11:49:48 mechanoid Exp mechanoid $
*       
*  multiscale retinex
*
*       OS : GNU/Linux 3.14.19 x86_64
* COMPILER : GNU project C and C++ Compiler  v. 4.9.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://www.mechanoid.kiev.ua
*  e-mail : cv@mechanoid.kiev.ua
* 
************************************************************/
#include <cv.h>
#include <highgui.h>
#include <stdio.h>

#define WIN_SRC   "ocv-example original"
#define WIN_FLT   "ocv-example filtered"
#define WIN_CTL   "ocv-example control"

IplImage* frame_c3[2] ;
IplImage* frame_c1[3];
IplImage* frame_c3f[3] ;
IplImage* frame_c1f[3] ;

int tb_value[4]={ 0, 0, 0, 9 };
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]);
int run(char* argv[]);
int done();

int multiscale_retinex( IplImage* src, IplImage* dst, double sigma, double* w );
void update(int a, void* b);

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main(int argc, char* argv[]) {
   if(!init(argc,argv)) { run(argv); }
   done();
   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int init(int argc, char* argv[]){
   int w;
   int h;
   int i;

   if(argc<2) { fprintf( stderr, "[E] bad parameters\n\n\t%s <in> \n",argv[0]); return 1; }

   frame_c3[0] = cvLoadImage( argv[1], CV_LOAD_IMAGE_COLOR);
   if(!frame_c3[0]){ fprintf( stderr, "[E] cvLoadImage: %s \n",argv[1]); return 3; }
   // реальные размеры картинки
   w=frame_c3[0]->width;
   h=frame_c3[0]->height;
   fprintf(stdout,"[i] %ix%i \n",w,h);

   frame_c3[1] = cvCloneImage(frame_c3[0]);
   if(!frame_c3[1]){ fprintf( stderr, "[E] cvCloneImage\n"); return 5; }		
  
   for(i=0;i<3;i++) {
	  frame_c1[i] = cvCreateImage(cvGetSize(frame_c3[0]),frame_c3[0]->depth,1);
	  if(!frame_c1[i]) { fprintf( stderr, "[E] cvCreateImage\n"); return 4; }		
   }

   for(i=0;i<3;i++) {
	  frame_c3f[i] = cvCreateImage(cvGetSize(frame_c3[0]), IPL_DEPTH_32F , frame_c3[0]->nChannels);
	  if(!frame_c3f[i]) { fprintf( stderr, "[E] cvCreateImage\n"); return 4; }		
	  cvZero(frame_c3f[i]);
   }

   for(i=0;i<3;i++) {
	  frame_c1f[i] = cvCreateImage(cvGetSize(frame_c3[0]), IPL_DEPTH_32F , 1);
	  if(!frame_c1f[i]) { fprintf( stderr, "[E] cvCreateImage\n"); return 4; }		
	  cvZero(frame_c1f[i]);
   }

   cvNamedWindow( WIN_SRC, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_SRC, 10,30 );

   cvNamedWindow( WIN_FLT, CV_WINDOW_AUTOSIZE ); 
   cvMoveWindow( WIN_FLT, 10+(10+w),30);

   cvNamedWindow( WIN_CTL, CV_WINDOW_NORMAL); 
   cvMoveWindow( WIN_CTL, 10,30+(30+h));

   cvCreateTrackbar("s: ", WIN_CTL, &tb_value[3], 20, (CvTrackbarCallback)update);
   cvCreateTrackbar("-R: ",WIN_CTL, &tb_value[2], 5, (CvTrackbarCallback)update);
   cvCreateTrackbar("-G: ",WIN_CTL, &tb_value[1], 5, (CvTrackbarCallback)update);
   cvCreateTrackbar("-B: ",WIN_CTL, &tb_value[0], 5, (CvTrackbarCallback)update);

   fprintf(stdout,"[i] press any key to exit...\n");


   return 0;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void update(int a, void* b){
   double weight[3]={ 1.0, 1.0, 1.0 };
   double sigma=tb_value[3];
   int i;

   for(i=0;i<3;i++) { weight[i]=1.0/((double)tb_value[i]+1.0); }
   sigma=(tb_value[3]<1) ? 0.1 : (double)tb_value[3] ;

   fprintf(stdout,"%f %f %f %f\n",weight[0],weight[1],weight[2],sigma);
    
   // получить кадр	 
   multiscale_retinex( frame_c3[0], frame_c3[1], sigma,weight );
   cvShowImage( WIN_FLT, frame_c3[1]); // отображаем результат
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int run(char* argv[]){ 
   
   cvShowImage( WIN_SRC, frame_c3[0]); // отображаем оригинал
   update(0,NULL);
   cvWaitKey(0);

   cvSaveImage(argv[2], frame_c3[1], 0 );


   return 0;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int done(){
   int i;

   cvDestroyAllWindows();

   for(i=0;i<2;i++) { cvReleaseImage( &frame_c3[i] ); }
   for(i=0;i<3;i++) { cvReleaseImage( &frame_c1[i] ); }
   for(i=0;i<3;i++) { cvReleaseImage( &frame_c3f[i] ); }
   for(i=0;i<3;i++) { cvReleaseImage( &frame_c1f[i] ); }

   return 0;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// коррекция неравномерного освещения - multiscale retinex
int multiscale_retinex( IplImage* src, IplImage* dst, double sig, double* w ) {
   int i;

   // R = w * ( log(I)-log(gauss(I)) )

   cvConvert(src,frame_c3f[0]); // конвертируем данные во float
   cvConvertScale(frame_c3f[0],frame_c3f[0],1.0, 1.0); // сдвигаем на 1 для вычисления логарифма
   cvLog(frame_c3f[0],frame_c3f[0]); // log(I)

   // сглаживаем и конвертируем данные во float
   cvSmooth( src,frame_c3[1],CV_GAUSSIAN,0,0,sig,0);  
   cvConvert(frame_c3[1],frame_c3f[1]); 
   cvConvertScale(frame_c3f[1],frame_c3f[1],1.0, 1.0); // сдвигаем на 1 для вычисления логарифма
   cvLog(frame_c3f[1],frame_c3f[1]); // log(gauss(I)) 

   cvSub(frame_c3f[0],frame_c3f[1],frame_c3f[0],NULL); // корректируем освещение
   cvExp( frame_c3f[0], frame_c3f[0] );

   cvSplit( frame_c3f[0], frame_c1f[0], frame_c1f[1], frame_c1f[2], NULL);
   for(i=0;i<3;i++) { 
	  cvNormalize(frame_c1f[i], frame_c1f[i], 0.0, 255.0, CV_MINMAX,NULL); // растягиваем в [0,255]
	  cvConvertScale(frame_c1f[i],frame_c1f[i], w[i], 0.0); // корректируем каналы
   }
   cvMerge(frame_c1f[0],frame_c1f[1],frame_c1f[2],NULL,frame_c3f[0]);

   cvConvert(frame_c3f[0],dst); 

   return 0;
}

